@@ -76,6 +76,7 @@ struct modem_data
   char                    *mms_apn;                             // The MMS APN
   char                    *MMS_proxy;                           // I *think* this is where mms proxy goes?
   char                    *default_number;
+  char                    *modem_number;
   // These are for settings the context (i.e. APN settings and if the bearer is active)
   char                    *context_interface;                   // Bearer interface here (e.g. "wwan0")
   char                    *context_path;                        // Dbus path of the bearer
@@ -105,6 +106,7 @@ struct modem_data
   gboolean                 auto_process_on_connection;
   gboolean                 autoprocess_sms_wap;
   gboolean                 get_all_sms_timeout;
+  gboolean                 modem_number_known;
 };
 
 typedef struct
@@ -241,6 +243,32 @@ dbus_change_settings (const char *key,
       save_string_settings (key_file, SETTINGS_GROUP, key,
                             val_str, &modem->default_number);
     }
+  else if (g_strcmp0 (key, "ModemNumber") == 0)
+    {
+      const char *service_modem_number = NULL;
+
+      service_modem_number = mms_service_get_own_number (modem->service);
+      if (modem->modem_number_known == TRUE)
+        {
+           g_warning ("Modem Number is known. Not setting ModemNumber");
+           return TRUE;
+        }
+      g_variant_get (value, "&s", &val_str);
+      if (!*val_str || g_strcmp0 (val_str, "NULL") == 0)
+        val_str = NULL;
+      if ((g_strcmp0 (val_str, service_modem_number) != 0) &&
+          (service_modem_number != NULL))
+        {
+           g_warning ("New Modem Number '%s' is different than current modem number '%s'",
+                      val_str, service_modem_number);
+           g_warning ("Now using number '%s'", val_str);
+        }
+      g_clear_pointer (&modem->modem_number, g_free);
+      modem->modem_number = g_strdup (val_str);
+      if (mms_service_set_own_number (modem->service, modem->modem_number))
+        save_string_settings (key_file, SETTINGS_GROUP, key,
+                              val_str, &modem->modem_number);
+    }
   else if (g_strcmp0 (key, "AutoProcessOnConnection") == 0)
     {
       g_variant_get (value, "b", &val_bool);
@@ -306,6 +334,7 @@ handle_method_call (GDBusConnection       *connection,
     {
       GVariantBuilder          settings_builder;
       GVariant                *settings, *all_settings;
+      const char *service_modem_number = NULL;
 
       g_variant_builder_init (&settings_builder, G_VARIANT_TYPE ("a{sv}"));
 
@@ -334,6 +363,20 @@ handle_method_call (GDBusConnection       *connection,
                                       "{'DefaultModemNumber', <%s>}",
                                       "NULL");
 
+      service_modem_number = mms_service_get_own_number (modem->service);
+
+      if (modem->modem_number)
+        g_variant_builder_add_parsed (&settings_builder,
+                                      "{'ModemNumber', <%s>}",
+                                      modem->modem_number);
+      else if (service_modem_number)
+        g_variant_builder_add_parsed (&settings_builder,
+                                      "{'ModemNumber', <%s>}",
+                                      service_modem_number);
+      else
+        g_variant_builder_add_parsed (&settings_builder,
+                                      "{'ModemNumber', <''>}");
+
       g_variant_builder_add_parsed (&settings_builder,
                                     "{'AutoProcessOnConnection', <%b>}",
                                     modem->auto_process_on_connection);
@@ -1225,10 +1268,15 @@ bearer_handler (mms_bool_t active,
   gint32 response;
   GDBusConnection *connection = mms_dbus_get_connection ();
   /* Check for any errors within the context */
+  g_strstrip (modem->mms_apn);
+  g_strstrip (modem->message_center);
+  if (modem->MMS_proxy != NULL)
+    g_strstrip (modem->MMS_proxy);
+
   response = set_context ();
   if (response != MMSD_MM_MODEM_CONTEXT_ACTIVE)
     {
-      DBG ("Set MMSC: %s, Set Proxy: %s, Set MMS APN: %s", modem->message_center, modem->MMS_proxy, modem->mms_apn);
+      DBG ("Set MMSC: '%s', Set Proxy: '%s', Set MMS APN: '%s'", modem->message_center, modem->MMS_proxy, modem->mms_apn);
       g_dbus_connection_emit_signal (connection,
                                      NULL,
                                      MMS_PATH,
@@ -1237,7 +1285,7 @@ bearer_handler (mms_bool_t active,
                                      g_variant_new ("(h)", response),
                                      NULL);
     }
-  DBG ("At Bearer Handler: path %s active %d context_active %d", modem->path, active, modem->context_active);
+  DBG ("At Bearer Handler: path '%s' active '%d' context_active '%d'", modem->path, active, modem->context_active);
   if (active == TRUE && modem->context_active == TRUE)
     {
       DBG ("active and context_active, bearer_notify");
@@ -1269,6 +1317,7 @@ static void
 find_settings_cb (const char *apn,
                   const char *mmsc,
                   const char *proxy,
+                  const char *mmssize,
                   GError     *error,
                   gpointer    user_data)
 {
@@ -1278,7 +1327,7 @@ find_settings_cb (const char *apn,
       g_warning ("Your MMS settings are not in the database! Please file a merge request at https://gitlab.gnome.org/GNOME/mobile-broadband-provider-info so they can be added");
     }
   else {
-      DBG ("Found settings: APN:%s, mmsc: %s, Proxy: %s", apn, mmsc, proxy);
+      DBG ("Found settings: APN:%s, mmsc: %s, Proxy: %s, MMS max size: %s", apn, mmsc, proxy, mmssize);
       if (mmsc != NULL)
         {
           GDBusConnection *connection = mms_dbus_get_connection ();
@@ -1309,6 +1358,12 @@ find_settings_cb (const char *apn,
                                  "CarrierMMSProxy",
                                  modem->MMS_proxy);
 
+          if (mmssize && *mmssize)
+            service_set_max_attach_size (modem->service,
+                                         atoi(mmssize));
+          else
+            service_set_max_attach_size (modem->service, DEFAULT_MAX_ATTACHMENT_TOTAL_SIZE);
+
           /* proxy will not be NULL here */
           g_dbus_connection_emit_signal (connection,
                                          NULL,
@@ -1384,9 +1439,10 @@ set_context (void)
                                                     find_settings_cb,
                                                     NULL);
             }
-          DBG ("Current Context APN: %s, mmsd-tng settings MMS APN: %s", apn, modem->mms_apn);
+          DBG ("Current Context APN: '%s', mmsd-tng settings MMS APN: '%s'", apn, modem->mms_apn);
           bearer_connected = mm_bearer_get_connected (modem_bearer);
-          if (g_strcmp0 (apn, modem->mms_apn) == 0)
+          /* APNs are not case sensitive: https://stackoverflow.com/questions/20834806/is-apn-name-case-sensitive */
+          if ((apn != NULL) && (modem->mms_apn != NULL) && (g_ascii_strcasecmp (apn, modem->mms_apn) == 0))
             {
               if (modem->state != MM_MODEM_STATE_CONNECTED)
                 {
@@ -1412,7 +1468,7 @@ set_context (void)
                   if (dns_ipv6)
                     dns_ipv6_csv = g_strjoinv (",", (gchar **)dns_ipv6);
                   mms_service_set_resolvers (modem->service, dns_ipv4_csv, dns_ipv6_csv);
-                  DBG ("DNS from modemmanager IPv6: %s, IPv4: %s", dns_ipv6_csv, dns_ipv4_csv);
+                  DBG ("DNS from modemmanager IPv6: '%s', IPv4: '%s'", dns_ipv6_csv, dns_ipv4_csv);
                   g_clear_pointer (&dns_ipv6_csv, g_free);
                   g_clear_pointer (&dns_ipv4_csv, g_free);
                 }
@@ -1490,6 +1546,15 @@ mm_save_settings (void)
     g_key_file_set_string (modem->modemsettings, imsi_hash,
                            "CarrierMMSProxy", "NULL");
 
+  if (modem->modem_number)
+    g_key_file_set_string (modem->modemsettings, imsi_hash,
+                           "ModemNumber", modem->modem_number);
+
+  g_key_file_set_integer (modem->modemsettings,
+                          imsi_hash,
+                          "TotalMaxAttachmentSize",
+                          service_get_max_attach_size (modem->service));
+
   mms_settings_sync (IDENTIFIER, SETTINGS_STORE, modem->modemsettings);
 }
 
@@ -1498,6 +1563,7 @@ mm_retrieve_settings (void)
 {
   g_autofree char *imsi_hash = NULL;
   g_autoptr(GError) error = NULL;
+  int max_mms_size = 0;
 
   imsi_hash = g_strdup_printf ("Old Settings: %u", g_str_hash (modem->imsi));
 
@@ -1526,6 +1592,28 @@ mm_retrieve_settings (void)
       return FALSE;
     }
 
+  modem->modem_number = g_key_file_get_string (modem->modemsettings,
+                                               imsi_hash,
+                                               "ModemNumber", &error);
+
+  if (error)
+    {
+      modem->modem_number = NULL;
+      g_clear_error (&error);
+    }
+
+  max_mms_size = g_key_file_get_integer (modem->modemsettings,
+                                         imsi_hash,
+                                         "TotalMaxAttachmentSize", &error);
+
+  if (error)
+    {
+      max_mms_size = DEFAULT_MAX_ATTACHMENT_TOTAL_SIZE;
+      g_clear_error (&error);
+    }
+
+  service_set_max_attach_size (modem->service, max_mms_size);
+
   return TRUE;
 }
 
@@ -1578,6 +1666,7 @@ mmsd_modem_available (void)
                   g_clear_pointer (&modem->message_center, g_free);
                   g_clear_pointer (&modem->mms_apn, g_free);
                   g_clear_pointer (&modem->MMS_proxy, g_free);
+                  g_clear_pointer (&modem->modem_number, g_free);
                   if (mm_retrieve_settings ())
                     DBG ("Reloading settings");
                   else {
@@ -1585,6 +1674,7 @@ mmsd_modem_available (void)
                       modem->message_center = g_strdup ("http://mms.invalid");
                       modem->mms_apn = g_strdup ("apn.invalid");
                       modem->MMS_proxy = g_strdup ("NULL");
+                      service_set_max_attach_size (modem->service, DEFAULT_MAX_ATTACHMENT_TOTAL_SIZE);
                     }
 
                   g_key_file_set_string (modem->modemsettings, SETTINGS_GROUP,
@@ -1596,6 +1686,14 @@ mmsd_modem_available (void)
                   g_key_file_set_string (modem->modemsettings, SETTINGS_GROUP,
                                          "CarrierMMSProxy", modem->MMS_proxy);
 
+                  if (modem->modem_number)
+                    g_key_file_set_string (modem->modemsettings, SETTINGS_GROUP,
+                                           "ModemNumber", modem->modem_number);
+                  else
+                    g_key_file_remove_key (modem->modemsettings, SETTINGS_GROUP,
+                                           "ModemNumber", NULL);
+
+
                   g_dbus_connection_emit_signal (connection,
                                                  NULL,
                                                  MMS_PATH,
@@ -1621,6 +1719,7 @@ mmsd_modem_available (void)
       else
         g_warning ("IMSI is NULL!");
 
+      modem->modem_number_known = FALSE;
       modem_number_ref = mm_modem_get_own_numbers (modem->modem);
       for (guint i = 0; modem_number_ref && modem_number_ref[i]; i++)
         {
@@ -1630,9 +1729,25 @@ mmsd_modem_available (void)
 
           if (number)
             {
+              if (modem->modem_number)
+                {
+                   g_warning ("mmsd-tng was provided a number '%s' but modem manager found a number '%s'",
+                              modem->modem_number, number);
+                   g_warning ("mmsd-tng is deleting the provided number '%s'", modem->modem_number);
+                   g_key_file_remove_key (modem->modemsettings, SETTINGS_GROUP,
+                                          "ModemNumber", NULL);
+                   mms_settings_sync (IDENTIFIER, SETTINGS_STORE, modem->modemsettings);
+                }
+              modem->modem_number_known = TRUE;
               mms_service_set_own_number (modem->service, number);
               break;
             }
+          else if (modem->modem_number)
+            {
+              DBG ("Could not find modem number, using number from settings");
+              mms_service_set_own_number (modem->service, modem->modem_number);
+              break;
+            }
         }
 
       gdbus_modem = MM_GDBUS_MODEM (modem->modem);
@@ -1668,6 +1783,7 @@ mmsd_modem_unavailable (void)
                                modem->modem_state_watch_id);
   g_object_unref (modem->sim);
   g_clear_pointer (&modem->mccmnc, g_free);
+  g_clear_pointer (&modem->modem_number, g_free);
   g_free (modem->imsi);
   g_free (modem->path);
   g_clear_object (&modem->modem);
@@ -1787,6 +1903,16 @@ modemmanager_init (void)
       g_clear_error (&error);
     }
 
+  modem->modem_number = g_key_file_get_string (modem->modemsettings,
+                                                 SETTINGS_GROUP,
+                                                 "ModemNumber", &error);
+  if (error)
+    {
+      DBG ("No Modem Number was configured.");
+      modem->modem_number = NULL;
+      g_clear_error (&error);
+    }
+
   mms_settings_close (IDENTIFIER, SETTINGS_STORE, modem->modemsettings, TRUE);
   modem->modemsettings = NULL;
   introspection_data = g_dbus_node_info_new_for_xml (introspection_xml_modemmanager, NULL);
@@ -1798,6 +1924,7 @@ modemmanager_init (void)
   modem->context_active = FALSE;
   modem->plugin_registered = FALSE;
   modem->get_all_sms_timeout = FALSE;
+  modem->modem_number_known = FALSE;
 
   modem->registration_id = g_dbus_connection_register_object (connection,
                                                               MMS_PATH,
